# coding:utf-8

from .attribute import __author__
from .attribute import __author_email__
from .attribute import __description__
from .attribute import __name__
from .attribute import __url_bugs__
from .attribute import __url_code__
from .attribute import __url_docs__
from .attribute import __url_home__
from .attribute import __version__
from .systemd import systemd_service

try:
    from .systemd import systemd_path
except Exception:
    pass



# coding:utf-8

from configparser import ConfigParser
import os
from typing import Optional
from typing import Tuple

allowed_dirs: Optional[Tuple[str, ...]] = None
try:
    from .path import sd_path
    allowed_dirs = sd_path.systemd_system_dirs
except Exception:
    pass


class sd_service:

    def __init__(self, config: ConfigParser):
        assert isinstance(config, ConfigParser), \
            f"unexpected type: {type(config)}"
        self.__config: ConfigParser = config

    @classmethod
    def read_string(cls, value: str) -> ConfigParser:
        config: ConfigParser = ConfigParser()
        config.optionxform = lambda option: option  # type: ignore
        config.read_string(value)
        return config

    @classmethod
    def from_string(cls, value: str) -> "sd_service":
        return sd_service(cls.read_string(value))

    @classmethod
    def from_file(cls, file: str) -> "sd_service":
        assert os.path.exists(file), f"'{file}' not found."
        assert os.path.isfile(file), f"'{file}' is not a regular file."
        return sd_service.from_string(open(file).read())

    def create_unit(self, path: str, unit: str,
                    allow_update: bool = False) -> None:
        assert isinstance(path, str), f"unexpected type: {type(path)}"
        assert isinstance(unit, str), f"unexpected type: {type(unit)}"

        if allowed_dirs is not None:
            assert path in allowed_dirs, f"'{path}' is not in {allowed_dirs}"
        assert os.path.isdir(path), f"'{path}' not exists or is file"

        def get_service_unit_name(s: str) -> str:
            return s if s.endswith(".service") else ".".join([s, "service"])

        file: str = os.path.join(path, get_service_unit_name(unit))
        if os.path.exists(file) and not allow_update:
            raise FileExistsError(f"'{file}' already exists.")

        with open(file, "w") as hdl:
            self.__config.write(hdl)



# coding:utf-8

from typing import List
from typing import Optional
from typing import Sequence

from xarg import add_command
from xarg import argp
from xarg import commands
from xarg import run_command

from ..utils import __description__
from ..utils import __name__
from ..utils import __url_home__
from ..utils import __version__

subs: List[add_command] = list()

try:
    from .path import add_cmd_path
    subs.append(add_cmd_path)
except Exception:
    pass


@add_command(__name__)
def add_cmd(_arg: argp):
    pass


@run_command(add_cmd, *subs)
def run_cmd(cmds: commands) -> int:
    return 0


def main(argv: Optional[Sequence[str]] = None) -> int:
    cmds = commands()
    cmds.version = __version__
    return cmds.run(
        root=add_cmd,
        argv=argv,
        description=__description__,
        epilog=f"For more, please visit {__url_home__}.")



# coding=utf-8

from setuptools import find_packages
from setuptools import setup

from xmanage.utils import __author__
from xmanage.utils import __author_email__
from xmanage.utils import __description__
from xmanage.utils import __name__
from xmanage.utils import __url_bugs__
from xmanage.utils import __url_code__
from xmanage.utils import __url_docs__
from xmanage.utils import __url_home__
from xmanage.utils import __version__

setup(
    name=__name__,
    version=__version__,
    description=__description__,
    url=__url_home__,
    author=__author__,
    author_email=__author_email__,
    project_urls={"Source Code": __url_code__,
                  "Bug Tracker": __url_bugs__,
                  "Documentation": __url_docs__},
    packages=find_packages(include=["xmanage*"]))
